#ifndef AD5522_H
#define AD5522_H
// Read / Write  bit[28] 
#define AD55_RD       0x10000000L   // Read = 1, Write = 0
// Channel select bits[27:24] PMU[3:0]
// For register reads, select CHR or only one channel
#define AD55_CHR      0x00000000L   // Channel select bits: All CH bits 0 for SCR, system control register 
#define AD55_CH0      0x01000000L   // Channel 0
#define AD55_CH1      0x02000000L
#define AD55_CH2      0x04000000L
#define AD55_CH3      0x08000000L
#define AD55_CHA      0x0F000000L   // All channels
// Register select for Control or DAC registers Mode bits [23:22] M[1:0] 
#define AD55_MCR      0x00000000L   // Mode = 00 for Both SCR and PMU registers
#define AD55_M_M      0x00400000L   // Mode = 01: FIN M register
#define AD55_M_C      0x00800000L   // Mode = 10: FIN C register
#define AD55_M_X      0x00C00000L   // Mode = 11: FIN X1 register

// SCR Clamp enable bits[21:18]
#define AD55_SCR_CL0  0x00040000L   // Channel 0
#define AD55_SCR_CL1  0x00080000L   // Channel 1
#define AD55_SCR_CL2  0x00100000L   // Channel 2
#define AD55_SCR_CL3  0x00200000L   // Channel 3
#define AD55_SCR_CLA  0x003C0000L   // All channels
// SCR Comparator Output enable bits [17:14] 
#define AD55_SCR_CO0  0x00004000L   // Channel 0
#define AD55_SCR_CO1  0x00008000L   // Channel 1
#define AD55_SCR_CO2  0x00010000L   // Channel 2
#define AD55_SCR_CO3  0x00020000L   // Channel 3
#define AD55_SCR_COA  0x0003C000L   // All channels
// Comparator Bias Enable bit[13] 
#define AD55_SCR_COB  0x00002000L 
// DUT GND  per channel: SCR Comparator Output enable bit [12] 
#define AD55_SCR_DGC  0x00001000L   
// Guard Alarm bit [11] 
#define AD55_SCR_GA   0x00000800L   
// Clamp Alarm bit [10] 
#define AD55_SCR_CA   0x00000400L   
// Internal 10K Sense resistor bit[9] 
#define AD55_SCR_I10  0x00000200L
// Guard Enable bit [8] 
#define AD55_SCR_GDE  0x00000100L 
// Measure Out Gain Control bits[7:6], G[1:0]
#define AD55_SCR_GA2  0x00000080L   // MV gain: 0.2m,  MI Gain: 10x, default is 00
// Thermal Shutdown Enable bit [5]
#define AD55_SCR_TSD  0x00000020L   //Temp shutdown enabled, default
// Temp Shutdown setting
#define AD55_SCR_T130 0x00000000L   // 130C, default
#define AD55_SCR_T120 0x00000008L   // 120C
#define AD55_SCR_T110 0x00000010L   // 110C
#define AD55_SCR_T100 0x00000018L   // 100C
// Latched Alarm bit [2]
#define AD55_SCR_LAT  0x00000004L   // default is 0 , OFF
// Bits [1:0] are unused. Set to 0

/* SCR Bits
  Temp alarm limit, enable
  Measure Out Gain control bits [7:6]  (fixed)
  Internal 10K sense bit [9]
  Guard Alarm bit [11]
  GLobal clamp and comparator bits

*/

/* PMU Bits 
  Channel Enable PMU bit [21]Measure mode pin PMU bits [14:13] 3 modes and OPEN
  I range PMU Bits [17:15],  0-4
  Channel Mode PMU bits [20:19] 3 modes
  Clamp Enable SCR bits [21:18] or PMU [9]
  Comparator Enable bit [8]
  Sys Force and Sense bits [11:10]
*/

// PMU Register bits
// Channel Enable bit[21]
#define AD55_PMU_CHEN 0x00200000L
// Force function bit [20:19]
#define AD55_PMU_FVCI 0x00080000L   // Force V, Clamp I
#define AD55_PMU_FICV 0x00100000L   // Force V, Clamp I
#define AD55_PMU_HIZ  0x00180000L   // HiZ 
// Bit [18] is reserved
// Current range bits [17:15]
#define AD55_PMU_IR0  0x00000000L   // +/-5uA
#define AD55_PMU_IR1  0x00008000L   // +/-20uA
#define AD55_PMU_IR2  0x00010000L   // +/-200uA
#define AD55_PMU_IR3  0x00018000L   // +/-2mA
#define AD55_PMU_IR4  0x00020000L   // Ext I range
// Measure function bits [14:13]
#define AD55_PMU_MIS  0x00000000L   // Measure I sense
#define AD55_PMU_MVS  0x00002000L   // Measure V sense
#define AD55_PMU_MTE  0x00004000L   // Measure Temperature 
#define AD55_PMU_MOF  0x00006000L   // Measure pin is OPEN
// Force Input amplifier bit [12]
#define AD55_PMU_FIG  0x00000000L   // Force input amp is GND
#define AD55_PMU_FID  0x00001000L   // Force Input Amp connected to DAC (default)
// System force and sense connections: SYS_SENSE and SYS_Force bits [11:10] 
#define AD55_PMU_SY0  0x00000000L   // SYS_FORCE and SYS_SENSE are HiZ (default)
#define AD55_PMU_SY1  0x00000800L   // SYS_FORCE is HiZ, SYS_SENSE is Measure V
#define AD55_PMU_SY2  0x00000400L   // SYS_FORCE is conn, SYS_SENSE is HiZ
#define AD55_PMU_SY3  0x00000C00L   // SYS_FORCE and SYS_SENSE are connected
// Clamp enable bit Replicated in the SCR bit [9]
#define AD55_PMU_CLE  0x00000200L   // Enable Clamp 
// Compare Output enable bit [8]
#define AD55_PMU_CLE  0x00000200L   // Enable Comparator
// Compare V or I bit [8]
#define AD55_PMU_COI  0x0000000L    // Compare Current I 
#define AD55_PMU_COV  0x0000040L    // Compare Voltage 
// Compare V or I bit [7] 
#define AD55_PMU_COM  0x00000080L   // SYS_FORCE and SYS_SENSE are HiZ (default)
// Clear or reset alarms, all 4 channels. 
#define AD55_PMU_CLR  0x00000040L   // Clear alarms on all channels
// Bits[5:0] are unused: Set to 0

// DAC register select bits [21:16]. Use with Mode bits to select DAC register
#define DAC_FI0      0x00080000L  // Current DAC range 0 +/- 5uA 
#define DAC_FI1      0x00090000L  // Current DAC range 0 +/- 20uA 
#define DAC_FI2      0x000A0000L  // Current DAC range 0 +/- 200uA
#define DAC_FI3      0x000B0000L  // Current DAC range 0 +/- 2mA
#define DAC_FI4      0x000C0000L  // Current DAC range 0 +/- Ext
#define DAC_FV0      0x000D0000L  // Voltage DAC
// Clamps
#define DAC_CLLI     0x00140000L  // Current clamp Low
#define DAC_CLLV     0x00150000L  // Voltage clamp Low 
#define DAC_CLHI     0x001C0000L  // Current clamp High 
#define DAC_CLHV     0x001D0000L  // Voltage clamp High
// High Compares
#define DAC_CCLI0    0x00200000L  // Current Compare Low range 0 +/- 5uA 
#define DAC_CCLI1    0x00210000L  // Current Compare Low range 0 +/- 20uA 
#define DAC_CCLI2    0x00220000L  // Current Compare Low range 0 +/- 200uA
#define DAC_CCLI3    0x00230000L  // Current Compare Low range 0 +/- 2mA
#define DAC_CCLI4    0x00240000L  // Current Compare Low range 0 +/- Ext
#define DAC_CCLV     0x00250000L  // Voltage Compare Low
//Low Compares
#define DAC_CCHI0    0x00280000L  // Current Compare High range 0 +/- 5uA 
#define DAC_CCHI1    0x00290000L  // Current Compare High range 0 +/- 20uA 
#define DAC_CCHI2    0x002A0000L  // Current Compare High range 0 +/- 200uA
#define DAC_CCHI3    0x002B0000L  // Current Compare High range 0 +/- 2mA
#define DAC_CCHI4    0x002C0000L  // Current Compare High range 0 +/- Ext
#define DAC_CCHV     0x002D0000L  // Voltage Compare High 

// AD5522 and ADC pins
#define AD5522_CS 10
#define ADC_CNV 11

// Struct defines 
// Measure Modes. Since each chan measure pins are tied together, one ADC, 
//  need to make 1 channel measure, the other 3 Hi-Z
#define MI 0   // Measure Current
#define MV 1   // Measure Voltage
#define MT 2   // Measure Temp
#define MHIZ 3 // Hi-Z
// I ranges
#define IR5UA   0 // I range 5uA
#define IR20UA  1 // I range 20uA
#define IR200UA 2 // I range 200uA
#define IR2MA   3 // I range 2mA
#define IR50MA  4 // I range 20mA


// AD5522 Globals
unsigned long PMU[4];   // Register shadows
unsigned long SCR; 

struct Fca{         // Force Cal
  unsigned int C;   // Offset
  unsigned int M;   // Gain
};

struct Mca{         // Measure Cal
  float O;          // Offset
  float G;          // Gain
};

#define IF0 0       // Range definitions for Cal, others
#define IF1 1 
#define IF2 2 
#define IF3 3 
#define IF4 4       // Max current range
#define VF  5        
#define CLLI 6
#define CLHI 7 
#define CLLV 8 
#define CLHV 9

#define NUM_CH 4
#define F_CAL_NUM 10    // 10 Force and 6 Measure Cal ranges
#define M_CAL_NUM 6     
#define F_CAL_SIZ 4 

#define FV 0            // Force Voltage
#define FI 1            // Force Current
#define FHIZ 2          // Force HiZ

struct  Fca Fcal [NUM_CH] [F_CAL_NUM];  // Force Cal array: 4 Channels, 10 ranges
struct  Mca Mcal [NUM_CH] [F_CAL_NUM];  // Measure Cal array

// Channel Data Struct
typedef struct {
  unsigned char IRange;   // Current range
  unsigned char FMode;    // FV, FI
  unsigned char MMode;    // 0: MI 1: MV: 2: MT 
  float forceV;
  float forceI;
  float clampVp;
  float clampVn;
  float clampIp;
  float clampIn;
  float measV;
  float measI;

 } ChanTypeDef;

ChanTypeDef Chan[4];

// VM Coarse calibrations
#define VM_C_O 29534.0
#define VM_C_G 26202.0
#define IM_C_O 29483.0
#define IM_C_G 55605.0-IM_C_O

// Global Instrument Struct
typedef struct {
  unsigned char MChan;    // Selected Measure channel
 } InstrTypeDef;

InstrTypeDef Instr;

// Range Struct        
typedef struct {
  char  *RangeUnit;     // Range units string
  char  *RangeLabel;    // Range label string  
  int   FOffsCal;       // Force Cal
  int   FGainCal;
  float MOffsCal;       // Measure Cal
  float MGainCal;
  float COffsPCal;      // Clamp + Cal
  float CGainPCal;
  float COffsNCal;      // Clamp - Cal
  float CGainNCal;
  float Mfull;          // Measure Full scale range
//  long  FDispMax;       // Max force and clamp display in integers
//  long  FDispMin;       // Min display in integers
//  long  FDispInit;      // Initial force display       
  long long  F_ItoF_Scale;   // Scale to convert Iset to float for force
  long  M_ItoF_Scale;   // Scale to convert Iset to float for force
  byte  ForceDP;        // Decimal point 0-5
  byte  MeasDP;         // Decimal point 0-5
  byte  ClampDP;        // Decimal point 0-5
  } rangeTypeDef;

#define NUM_V_RANGES 1
#define NUM_I_RANGES 5
#define NUM_RANGES NUM_V_RANGES+NUM_I_RANGES
#define IRange_5uA  0
#define IRange_20uA 1
#define IRange_200uA 2
#define IRange_2mA  3
#define IRange_10mA 4
#define VRange_10V   5

rangeTypeDef Range[NUM_RANGES];   // Create structs for each range. Should  make separate I and V ranges ?

#endif    // AD5522_H

