/* Command interface. Commands are single ASCII characters, and can be 
   from the serial keyboard, SCPI interface, key / encoders, GUI touch screen, or anywhere. 
   a large 'case' statement is used to decode commands into actions. 
*/

/* Key map. Convert key number to commands.  */
// Available: a,b, , ,e,f,g,h,i,j,k, , ,n,o,p,q,r,s,t, ,v,w,x,y,z <>,.:;"'{}[]()
const char keyMap[16] = {
  0,    // 0: EncA
  0,    // 1: EncB
  'k',  // 2: Enc Sw
  'u',  // 3: Up
  'l',  // 4: Left
  'r',  // 5: Right
  'd',  // 6: Down
  '0',  // 7: Ch 0 
  '1',  // 8: Ch 1
  '2',  // 9: Ch 2
  '3',  // 10: Ch 3
  'c',  // 11: Unused for now  No, Cal 
  'C',  // 12: Unused for now 
  'm',  // 13: Unused for now 
  'M',  // 14: Unused for now 
  'e',  // 15: Enter (center)
};


 void getCommand(void){
  int i;
 // char incomingByte;
  // Get control, act on it.
  while (Serial.available() > 0)
  {
    // A while loop here would read faster than one byte per loop.
    // read the incoming byte:
    command = Serial.read(); // read one byte from serial
    Serial.print("Ser: "); Serial.println(command);
    execCommand();
  }

  //Get Encoder changes.
  while (encCount != oldEncCount)  {
    if (encCount > oldEncCount)     {
      command = 'u';
  //    Serial.println("Enc: u");
      execCommand(); 
      oldEncCount ++;
    }
    if (encCount < oldEncCount) {
      command = 'd';
  //    Serial.println("Enc: d");
      execCommand(); 
      oldEncCount --;
    }
    
  }

  oldEncCount = encCount;

  // Check all the Key Flags
  for (i = 0; i < 16; i++){
    if (keyFlag[i]) {
      // Serial.print("Key = ");
      // Serial.println(i);
      keyFlag[i] = 0;
      command = keyMap[i];    // Look up flag to command
      execCommand();
    }
  }

  // key = getKeypad();
  // if (key)
  // {
  //   command = key;
  //   Serial.print("Key: ");
  //   Serial.println(command);
  // }
}

void execCommand(void){
  int inc;
  int i;
  // if(commmand) {
  //   Serial.print("Command = "); Serial.println(command);
  // }
  switch (command)
  {
    //     case 'a':      //a n  nnnnn Set DAC n in counts
    //       i = Serial.parseInt();    // Get DAC number
    //       dac[i] = Serial.parseInt();  //Careful, with ParseInt() if there is no
    //               //non-numeric character after int,  serial waits for a timeout (1s default)
    //       break;
    
    case 'h':        // Help 
      Serial.println("Help Menu");
      Serial.println("0,1,2,3: set channel, ud: V up / down 1.000V");
      Serial.println("Xzx: set 10V / 0V / -10V");
      Serial.println("Cal; Mm: inc/dec M, Cc: inc/dec C, p: print");
      Serial.println("i: Init EEPROM, e: read EEPROM, ");
      Serial.println("Ff: Force V/I, ~!@#$: Irange 0-4");
      break;
    
    // Increase / decrease Cal factors C (offset) and M (Gain)
    case 'C':       // Increase offset C, Set to global V with Cal factors
      Fcal[ch][VF].C += 1;
      AD5522SetVoltCal(ch, Chan[ch].forceV);
      Serial.println(Fcal[ch][VF].C, HEX);
      break;
    case 'c':       // Decrease offset C, Set to global V with Cal factors
      Fcal[ch][VF].C -= 1;
      AD5522SetVoltCal(ch, Chan[ch].forceV);
      Serial.println(Fcal[ch][VF].C, HEX);
      break;
    case 'M':       // Increase gain M, Set to global V with Cal factors
      Fcal[ch][VF].M += 1;
      if (Fcal[ch][VF].M >= 0xFFFF) Fcal[ch][VF].M = 0xFFFF;
      AD5522SetVoltCal(ch, Chan[ch].forceV);
      Serial.println(Fcal[ch][VF].M, HEX);
      break;
    case 'm':       // Decrease offset C, Set to global V with Cal factors
      Fcal[ch][VF].M -= 1;
      AD5522SetVoltCal(ch, Chan[ch].forceV);
      Serial.println(Fcal[ch][VF].M, HEX);
      break; 

    // Set Force values
    case 'X':       // Set V or I to Max full scale
      if (Chan[ch].FMode == FV) {
        Chan[ch].forceV = 10.0;
        AD5522SetVoltCal(ch, Chan[ch].forceV);
        Serial.println(Chan[ch].forceV);  
      } 
      if  (Chan[ch].FMode == FI) { 
        Chan[ch].forceI = 0.002;
        AD5522SetCurr(ch, Chan[ch].forceI);
        Serial.println(Chan[ch].forceI,6);  
      }
      break;
    case 'z':       // Set V  or I to 0.0 with cal
      if (Chan[ch].FMode == FV) {
        Chan[ch].forceV = 0.0;
        AD5522SetVoltCal(ch, Chan[ch].forceV);
        Serial.println(Chan[ch].forceV);  
      } 
      if  (Chan[ch].FMode == FI) { 
        Chan[ch].forceI = 0.000;
        AD5522SetCurr(ch, Chan[ch].forceI);
        Serial.println(Chan[ch].forceI,6);  
      }
      break;
    case 'x':       // Set V to Min -10.0 with cal
       if (Chan[ch].FMode == FV) {
        Chan[ch].forceV = -10.0;
        AD5522SetVoltCal(ch, Chan[ch].forceV);
        Serial.println(Chan[ch].forceV);  
      } 
      if  (Chan[ch].FMode == FI) { 
        Chan[ch].forceI = -0.002;
        AD5522SetCurr(ch, Chan[ch].forceI);
        Serial.println(Chan[ch].forceI,6);  
      }   
      break;
    case 'u':      
//      Serial.println("u");
      if (menu == 0){   // Force Menu, underline force
        inc = 1; // Scale inc by cursor position
        for( i=0; i < disp[1].cursor; i++) inc *= 10;
        Chan[ch].forceV += 0.001 * inc;
        if (Chan[ch].forceV > 11.0) Chan[ch].forceV = 11.0;
        AD5522SetVoltCal(ch, Chan[ch].forceV);
        Serial.println(Chan[ch].forceV);      
      }
      if (menu == 1){  // Range Menu
        Chan[ch].IRange ++;
        if (Chan[ch].IRange > IF4){
          Chan[ch].IRange = IF4;          
        }
        AD5522SetIRange(ch, Chan[ch].IRange);
        Serial.print( "I range: ");
        Serial.println(Chan[ch].IRange);
      }
      if (menu == 2){  // Up is FV
        AD5522SetFMode(ch, FV);
        Serial.println("FV");
      }
      break;
    case 'd':    
//      Serial.println("d");
      if (menu == 0){   // Force Menu, underline force
        inc = 1; // Scale inc by cursor position
        for( i=0; i < disp[1].cursor; i++) inc *= 10;
        Chan[ch].forceV -= 0.001 * inc;
        if (Chan[ch].forceV < -11.0) Chan[ch].forceV = -11.0;
        AD5522SetVoltCal(ch, Chan[ch].forceV);
        Serial.println(Chan[ch].forceV);     
      }
      if (menu == 1){  // I Range Menu
        Chan[ch].IRange --;
        if ( !(Chan[ch].IRange >= 0 && Chan[ch].IRange <= IF4)){   // limit range select
          Chan[ch].IRange = 0;
        }
        AD5522SetIRange(ch, Chan[ch].IRange);
        Serial.print( "I range: ");
        Serial.println(Chan[ch].IRange);
      }  
      if (menu == 2){  // Mode Menu, Down is FI
        AD5522SetFMode(ch, FI);
        Serial.println("FI");
      }  
      break;
    // Channel selections
    case '0':      // Set channel 0
      ch = 0;
      Serial.print("ch = "); Serial.println(ch);
      break;
    case '`':      // Set channel 0, alt.
      ch = '`';
      Serial.print("ch = "); Serial.println(ch);
      break;
    case '1':      // channel 1
      ch = 1;
      Serial.print("ch = "); Serial.println(ch);
      break;
    case '2':      // channel 2
      ch = 2;
      Serial.print("ch = "); Serial.println(ch);
      break;
    case '3':      // channel 3
      ch = 3;
      Serial.print("ch = "); Serial.println(ch);
      break;
  
  // EEPROM actions
    case 'p':      
      Serial.println("Print Cal");
      printCal(ch);
      break; 
    case 'e':              // Read EEPROM 
      Serial.println("Read EEPROMe");
      EEFPrint (ch);
      break;
    case 'i':              // Set Force Init EEPROM values for 1 channel
      Serial.println("Init EEPROM");
      EEFInit(ch);
      break;
  // Set force mode 
    case  'F': 
      Serial.println("FV");
      AD5522SetFMode(ch, FV);
      printPMU();
      break;
    case  'f': 
      Serial.println("FI");
      AD5522SetFMode(ch, FI);
      printPMU();
      break;    
  // Current range
    case '~':      // Set irange 0
      Serial.println("IR = 5uA ");
      AD5522SetIRange(ch, 0);
      break;
    case '!':      // Set irange 1
      Serial.println("IR = 20uA ");
      AD5522SetIRange(ch, 1);
      break;
    case '@':      // Set irange 2
      Serial.println("IR = 200uA ");
      AD5522SetIRange(ch, 2);
      break;
    case '#':      // Set irange 3 
      Serial.println("IR = 2mA ");
      AD5522SetIRange(ch, 3);
      break;
    case '$':      // Set irange 4 
      Serial.println("IR = 50mA ");
      AD5522SetIRange(ch, 4);
      break;

    case 'l':    // move digit left
      disp[1].cursor ++;
      if (menu == 0){   // Force Menu, underline force
        if(disp[1].cursor > 3) disp[1].cursor = 3;
        }
      Serial.println("l");
      break;
    case 'r':
      disp[1].cursor --;
      if(disp[1].cursor <= 0) disp[1].cursor = 0;
      Serial.print("r");
      //Serial.println(disp[1].cursor);
      break;
    case 'k':           // Encoder Sw controls memus
      menu ++;
      if( menu > menuMax ) menu = 0;    // Wrap around
      Serial.print("k ");
      Serial.println(menu);

      break;

    default:
      break;
  }
  command = 0;
}
